module de0(

    // Reset
    input              RESET_N,

    // Clocks
    input              CLOCK_50,
    input              CLOCK2_50,
    input              CLOCK3_50,
    inout              CLOCK4_50,

    // DRAM
    output             DRAM_CKE,
    output             DRAM_CLK,
    output      [1:0]  DRAM_BA,
    output      [12:0] DRAM_ADDR,
    inout       [15:0] DRAM_DQ,
    output             DRAM_CAS_N,
    output             DRAM_RAS_N,
    output             DRAM_WE_N,
    output             DRAM_CS_N,
    output             DRAM_LDQM,
    output             DRAM_UDQM,

    // GPIO
    inout       [35:0] GPIO_0,
    inout       [35:0] GPIO_1,

    // 7-Segment LED
    output      [6:0]  HEX0,
    output      [6:0]  HEX1,
    output      [6:0]  HEX2,
    output      [6:0]  HEX3,
    output      [6:0]  HEX4,
    output      [6:0]  HEX5,

    // Keys
    input       [3:0]  KEY,

    // LED
    output      [9:0]  LEDR,

    // PS/2
    inout              PS2_CLK,
    inout              PS2_DAT,
    inout              PS2_CLK2,
    inout              PS2_DAT2,

    // SD-Card
    output             SD_CLK,
    inout              SD_CMD,
    inout       [3:0]  SD_DATA,

    // Switch
    input       [9:0]  SW,

    // VGA
    output      [3:0]  VGA_R,
    output      [3:0]  VGA_G,
    output      [3:0]  VGA_B,
    output             VGA_HS,
    output             VGA_VS
);

// Z-state
assign DRAM_DQ = 16'hzzzz;
assign GPIO_0  = 36'hzzzzzzzz;
assign GPIO_1  = 36'hzzzzzzzz;

// LED OFF
assign HEX0 = 7'b1111111;
assign HEX1 = 7'b1111111;
assign HEX2 = 7'b1111111;
assign HEX3 = 7'b1111111;
assign HEX4 = 7'b1111111;
assign HEX5 = 7'b1111111;

// Генерация частот
wire locked;
wire clock_25;
wire clock_100;

// Провода к процессору
wire [12:0] address_vga;
wire [ 7:0] data;
wire [19:0] address;
wire [ 7:0] o_data;
wire        we;

// Выбор памяти
wire        we_bios = address >= 20'hF0000;
wire        we_vga  = address >= 20'hB8000 && address < 20'hBA000;

// Выбор данных из памяти (роутинг)
wire [ 7:0] i_data_bios;
wire [ 7:0] i_data_vga;

wire [ 7:0] i_data =
    we_bios ? i_data_bios :
    we_vga  ? i_data_vga : 8'h00;

de0pll unit_pll
(
    .clkin     (CLOCK_50),
    .m25       (clock_25),
    .m100      (clock_100),
    .locked    (locked)
);

// ---------------------------------------------------------------------

cpu CPU8088
(
    .clock      (clock_25),
    .reset_n    (1'b1),
    .locked     (locked),
    .address    (address),
    .i_data     (i_data),
    .o_data     (o_data),
    .we         (we)
);

// ---------------------------------------------------------------------

vga VgaUnit
(
    .clock      (clock_25),
    .r          (VGA_R[3:2]),
    .g          (VGA_G[3:2]),
    .b          (VGA_B[3:2]),
    .hs         (VGA_HS),
    .vs         (VGA_VS),
    .address    (address_vga),
    .data       (data)
);

// ---------------------------------------------------------------------

// 8k: Видеопамять и знакогенератор
font UnitFont
(
    .clock      (clock_100),
    .address_a  (address_vga),
    .q_a        (data),
    .address_b  (address[12:0]),
    .q_b        (i_data_vga),
    .data_b     (o_data),
    .wren_b     (we & we_vga),
);

// 64K: BIOS
bios BiosMemory
(
    .clock      (clock_100),
    .address_a  (address[15:0]),
    .q_a        (i_data_bios),
    .data_a     (o_data),
    .wren_a     (we & we_bios)
);

endmodule

`include "../vga.v"
`include "../cpu.v"
